﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Example6_UsingEncoders
{
    class Program
    {
        // Encoder options
        public enum encoderOptions
        {
            encoderEnabled   = 1 << 0,
            enable4xSampling = 1 << 1,
            directMappingA   = 1 << 4,
            macroMappingA    = 1 << 5,
            directMappingB   = 1 << 6,
            macroMappingB    = 1 << 7
        }

        // Key modifiers
        public enum keyModifiers
        {
            CtrlMask = 1,
            ShiftMask = 2,
            AltMask = 4,
            WinMask = 8,
            AltGrMask = 64
        }

        // Pin functions
        public enum pinFunctionsEnum
        {
            pinFunctionInactive = 0,
            pinFunctionDigitalInput = 2,
            pinFunctionDigitalOutput = 4,
            pinFunctionAnalogInput = 8,
            pinFunctionAnalogOutput = 16,
            pinInvert = 128
        }

        static void Main(string[] args)
        {
            Console.WriteLine("PoKeys55 Example 6 - using encoder switches\n");

            // Create PoKeys55 device object
            PoKeysDevice_DLL.PoKeysDevice device = new PoKeysDevice_DLL.PoKeysDevice();

            // Enumerate devices (this step must never be ommited!)
            int iNumDevices = device.EnumerateDevices();

            if (iNumDevices == 0)
            {
                Console.WriteLine("No PoKeys55 devices found!");


                Console.WriteLine("\nPress any key to exit");
                Console.ReadKey();
                return;
            }

            // Connect to first PoKeys55 device
            device.ConnectToDevice(0);

            Console.WriteLine("\nSetting encoder 1 to use pins 1 and 2...");
            // Let's configure encoder 1 to use pins 1 (channel A) and 2 (channel B) with no key mappings
            // Pins 1 and 2 must be set as digital inputs at first
            device.SetPinData(0, (byte)pinFunctionsEnum.pinFunctionDigitalInput);
            device.SetPinData(1, (byte)pinFunctionsEnum.pinFunctionDigitalInput);
            device.SetEncoderSettings(0, (byte)encoderOptions.encoderEnabled, 0, 1);

            Console.WriteLine("Setting encoder 2 to use pins 3 and 4...");
            // and encoder 2 to use pins 3 (channel A) and 4 (channel B) with direct key mapping
            // Pins 3 and 4 must also be set as digital inputs at first
            device.SetPinData(2, (byte)pinFunctionsEnum.pinFunctionDigitalInput);
            device.SetPinData(3, (byte)pinFunctionsEnum.pinFunctionDigitalInput);
            device.SetEncoderSettings(1, (byte)encoderOptions.encoderEnabled + 
                        (byte)encoderOptions.directMappingA + (byte)encoderOptions.directMappingB, 2, 3);
            // Let direction A be mapped to up key and direction B to shift + down key
            device.SetEncoderKeyMappingDirA(1, 0x52, 0);
            device.SetEncoderKeyMappingDirB(1, 0x51, (byte)keyModifiers.ShiftMask);

            Console.WriteLine("Setting encoder 3 to use pins 5 and 6...");
            // and encoder 3 to use pins 5 (channel A) and 6 (channel B) in 4x sampling mode
            // Pins 5 and 6 must also be set as digital inputs at first
            device.SetPinData(4, (byte)pinFunctionsEnum.pinFunctionDigitalInput);
            device.SetPinData(5, (byte)pinFunctionsEnum.pinFunctionDigitalInput);
            device.SetEncoderSettings(2, (byte)encoderOptions.encoderEnabled + (byte)encoderOptions.enable4xSampling, 4, 5);

            Console.WriteLine("\nEntering display loop... press Escape to quit");

            // Reset encoder 1 first
            device.ResetEncoderValue(0);
            
            // Now let's create a loop and read the raw values of the encoders
            ConsoleKeyInfo cki; 
            int encoder1Value = 0;
            int encoder2Value = 0;
            int encoder3Value = 0;

            while (true)
            {
                if (Console.KeyAvailable == true)
                {
                    cki = Console.ReadKey(); 

                    if (cki.Key == ConsoleKey.Escape) break;
                }

                // Get RAW values from device
                device.GetEncoderRAWValue(0, ref encoder1Value);
                device.GetEncoderRAWValue(1, ref encoder2Value);
                device.GetEncoderRAWValue(2, ref encoder3Value);

                Console.Write("\rEncoder 1 value: " + encoder1Value + "    Encoder 2 value: " + encoder2Value + "    Encoder 3 value: " + encoder3Value + "       ");
            }

            device.DisconnectDevice();

            Console.WriteLine("\nPress any key to exit");
            Console.ReadKey();
        }
    }
}
